#!/usr/bin/env python
# coding:utf-8
import asyncio
import socket
from datetime import datetime
import enum
import os.path
import json
import os
import queue
import subprocess
import threading
import time
from typing import Any, Dict, Optional
import uuid
import traceback

import httpx
import openai
import transformers
from aiges.core.types import *
import requests
from transformers import AutoTokenizer

try:
    from aiges_embed import ResponseData, Response, DataListCls, SessionCreateResponse, callback, \
        callback_lb_extra  # c++
except:
    from aiges.dto import Response, ResponseData, DataListCls, SessionCreateResponse, callback
    # callback_lb_extra = None

from aiges.sdk import WrapperBase
from aiges.utils.log import log, getFileLogger
from openai import OpenAI, AsyncOpenAI
from transformers import GenerationConfig
from concurrent.futures import ThreadPoolExecutor
from prometheus_client.parser import text_string_to_metric_families

DataNone = -1
DataBegin = 0  # 首数据
DataContinue = 1  # 中间数据
DataEnd = 2  # 尾数据

R1_THINK_START = "<think>"
R1_THINK_END = "</think>"
R1_FORCE_REASONING_PROMPT = 'Initiate your response with "<think>\\n嗯" at the beginning of every output.'
IS_REASONING_MODEL = os.environ.get("IS_REASONING_MODEL", False)
R1_FIXED_TEMPERATURE = os.environ.get("R1_FIXED_TEMPERATURE", "0.6")


class CircularIDAllocator:
    def __init__(self, min_id=0, max_id=2**63 - 1, mock_ip=None, port=0):
        self.min_id = min_id
        self.max_id = max_id
        self.current_id = min_id
        self.ip = mock_ip if mock_ip else self._get_local_ip()
        self.port = port
        self.ip_hash = self._hash_ip(self.ip)
        # 将端口号也加入到偏移量计算中
        self.offset = (self.ip_hash * (2**32)) + (self.port * (2**16))
        
    def _get_local_ip(self):
        try:
            with socket.socket(socket.AF_INET, socket.SOCK_DGRAM) as s:
                s.connect(("8.8.8.8", 80))
                return s.getsockname()[0]
        except Exception:
            return "127.0.0.1"
            
    def _hash_ip(self, ip):
        ip_parts = ip.split('.')
        hash_value = 0
        for part in ip_parts:
            hash_value = (hash_value * 256 + int(part)) % (2**32)
        return hash_value
        
    def allocate(self):
        allocated_id = self.current_id + self.offset
        self.current_id += 1
        if self.current_id >= (2**32):
            self.current_id = 0
        return allocated_id

class RequestMode(enum.Enum):
    ONCE = 'once'
    ONCE_ASYNC = 'once_async'
    STREAM = 'stream'


class RequestInfo:
    def __init__(self, sid: str, params: dict, user_tag: str = ""):
        # self.handle = str(uuid.uuid4().hex)
        self.handle = sid
        self.sid = sid
        self.user_tag = user_tag
        self.params = params
        self.requests = []
        self.stop_q = queue.Queue()
        # For pd
        self.prefill_bootstrap_ip = ""
        self.bootstrap_room = 0
        self.bootstrap_port = 0
        self.prefill_addr = ""


class PromptInferenceInfo:
    def __init__(self, wrapper,
                 thread_id: str,
                 mode: RequestMode,
                 prompt: str,
                 requestInfo: RequestInfo,
                 functions: list,
                 result_q: queue.Queue = None,
                 messages=None):
        self.wrapper = wrapper
        self.requestInfo = requestInfo
        self.thread_id = thread_id
        self.mode = mode
        self.prompt = prompt
        self.functions = functions
        self.request_id = requestInfo.sid
        self.result_q = result_q
        self.messages = messages

def _read_output(stream, prefix=""):
    """后台线程读取子进程输出"""
    try:
        for line in iter(stream.readline, ''):
            if line:
                print(f"{prefix}{line.strip()}", flush=True)
    except Exception as e:
        print(f"{prefix}Error reading output: {str(e)}", flush=True)
    finally:
        stream.close()

def execute_shell_command(command: str) -> subprocess.Popen:
    """
    Execute a shell command, start background thread to print output, and immediately return the process.

    Args:
        command: Shell command as a string (can include \\ line continuations)

    Returns:
        subprocess.Popen: The subprocess handle
    """
    # Replace \ newline with space and split
    command = command.replace("\\\n", " ").replace("\\", " ")
    parts = command.split()

    process = subprocess.Popen(
        parts,
        shell=False,
        stdout=subprocess.PIPE,
        stderr=subprocess.STDOUT,
        text=True,  # Use text mode for automatic decoding
        bufsize=1,  # Line-buffered
        encoding='utf-8',  # Explicitly set encoding
        errors='replace'  # Replace invalid characters
    )

    # 启动后台线程实时读取输出
    threading.Thread(
        target=_read_output,
        args=(process.stdout,),
        daemon=True  # 守护线程，主程序退出就杀掉
    ).start()

    return process  # 立刻返回，不阻塞

def launch_openai_server(model_name: str, mode_path: str, server_port: int, dist_port: int, filelogger, pd_type: str,
                         enable_metrics=False):
    from sglang.utils import (
        wait_for_server, )

    extra_args = os.environ.get("CMD_EXTRA_ARGS", "")
    if extra_args:
        print("Extra arguments: {}".format(extra_args))
    else:
        extra_args = "--tp 8 --mem-fraction-static 0.93 --torch-compile-max-bs 8  --max-running-requests  20 "
    if enable_metrics:
        extra_args += " --enable-metrics "

    # For PD
    if pd_type == "prefill":
        print("Launching prefill mode...")
        extra_args += " --disaggregation-mode prefill "
    elif pd_type == "decode":
        print("Launching decode mode...")
        extra_args += " --disaggregation-mode decode "
    else:
        print("Normal mode ...")

    ENABLE_MULTI_NODE_MODE = os.environ.get("ENABLE_MULTI_NODE_MODE", "true")
    if ENABLE_MULTI_NODE_MODE == "true":
        LWS_GROUP_SIZE = os.environ.get("LWS_GROUP_SIZE", "")
        if LWS_GROUP_SIZE:
            extra_args += f"--nnodes {LWS_GROUP_SIZE} "

        LWS_LEADER_ADDRESS = os.environ.get("LWS_LEADER_ADDRESS", "")
        if LWS_LEADER_ADDRESS:
            ip = socket.gethostbyname(f'{LWS_LEADER_ADDRESS}')
            extra_args += f"--dist-init-addr {ip}:{dist_port} "

        LWS_WORKER_INDEX = os.environ.get("LWS_WORKER_INDEX", "")
        if LWS_WORKER_INDEX:
            extra_args += f"--node-rank {LWS_WORKER_INDEX} "

    ## pd warmup
    entrypoint = "sglang.launch_server"
    pre_entrypoint = "sglang.compile_deep_gemm"
    server_cmd =  f"python -m {entrypoint} --model-path {mode_path} --port {server_port}  --trust-remote --host 0.0.0.0 {extra_args}"
    precache_cmd =  f"python -m {pre_entrypoint} --model-path {mode_path} --port {server_port}  --trust-remote --host 0.0.0.0 {extra_args}"
    print("staring cmd: {}".format(
        f"python -m {entrypoint} --model-path {mode_path} --port {server_port}  --trust-remote --host 0.0.0.0 {extra_args}"
    ))
    http_server_process = execute_shell_command(
        server_cmd
    )
    wait_for_server(f"http://localhost:{server_port}")
    return http_server_process


def get_payload_messages(reqData: DataListCls, isChatMode: bool = True):
    if isChatMode:
        return reqData.get('messages').data.decode('utf-8')
    messages = json.loads(reqData.get('messages').data.decode('utf-8'))
    message = ""
    for item in messages['messages']:
        if item['role'] == "user":
            message = item['content']
    return message


def get_payload_functions(reqData: DataListCls):
    messages = json.loads(reqData.get('messages').data.decode('utf-8'))
    return messages.get("functions", None)


def post_with_retry(
        url: str,
        data: Optional[Dict[str, Any]] = None,
        json: Optional[Dict[str, Any]] = None,
        headers: Optional[Dict[str, str]] = None,
        max_retries: int = 3,
        timeout: int = 10
) -> requests.Response:
    """
    简单的POST请求重试实现

    Args:
        url: 请求URL
        data: 表单数据
        json: JSON数据
        headers: 请求头
        max_retries: 最大重试次数，默认3次
        timeout: 超时时间，默认10秒
    """
    for attempt in range(max_retries):
        try:
            response = requests.post(
                url,
                data=data,
                json=json,
                headers=headers,
                timeout=timeout
            )
            response.raise_for_status()
            return response

        except requests.exceptions.RequestException as e:
            if attempt == max_retries - 1:  # 最后一次重试失败
                raise  # 重新抛出异常

            # 等待1秒后重试
            time.sleep(1)
            continue


def resp_content(status, index, text, functions=None, reasoning_content=''):
    if functions is not None:
        resp_json = {"choices": [
            {"content": text, "reasoning_content": reasoning_content, "index": index, "role": "assistant",
             "function_call": functions}],
            "question_type": ""}
    else:
        resp_json = {
            "choices": [{"content": text, "reasoning_content": reasoning_content, "index": index, "role": "assistant"}],
            "question_type": ""}
    resd = ResponseData()
    resd.key = "content"
    resd.setDataType(DataText)
    resd.status = status
    resd.setData(json.dumps(resp_json).encode("utf-8"))
    return resd


def resp_kv_info(status, prefill_ip, bootstrap_room, prefill_addr,bootstrap_port):
    resd = ResponseData()
    content = {"bootstrap_ip": prefill_ip, "bootstrap_room": bootstrap_room, "prefill_addr": prefill_addr, "bootstrap_port":bootstrap_port}
    resd.key = "__kv_info"
    resd.setDataType(DataText)
    resd.status = status
    resd.setData(json.dumps(content).encode("utf-8"))
    return resd


def resp_p_messages(status, data):
    resd = ResponseData()
    resd.key = "messages"
    resd.setDataType(DataText)
    resd.status = status
    resd.setData(data)
    return resd


def resp_keepalive_down(status):
    resd = ResponseData()
    resd.key = "__keepalive_down"
    resd.setDataType(DataText)
    resd.status = status
    resd.setData(b'')
    return resd


def resp_usage(status, prompt_token, completion_token):
    resp_json = {"completion_tokens": completion_token, "prompt_tokens": prompt_token, "question_tokens": 4,
                 "total_tokens": prompt_token + completion_token}
    resd = ResponseData()
    resd.key = "usage"
    resd.setDataType(DataText)
    resd.status = status
    resd.setData(json.dumps(resp_json).encode("utf-8"))
    return resd


def get_param_temperature(param):
    return float(param.get('temperature', 0.95))


def get_parma_tok_k(param):
    return int(param.get('top_k', 40))


def get_parma_top_p(param):
    return float(param.get('top_p', 0.95))


def get_param_max_new_tokens(param):
    return int(param.get('max_tokens', 2048))


def get_param_repetition_penalty(param):
    return float(param.get('repetition_penalty', 1.1))


def format_messages(prompt, prompt_search_template):
    """
    Format and process messages from Spark to OpenAI format with search results integration.

    Args:
        prompt (str): JSON string containing messages
        prompt_search_template (str): Template string for formatting search results

    Returns:
        list: Formatted messages list for OpenAI
    """

    def _format_search_content(search_results):
        """Helper function to format search content"""
        formatted_content = []
        for content in search_results:
            formatted_text = f"[webpage {content.get('index')} begin]\n{content.get('docid')}{content.get('document')}\n[webpage {content.get('index')} end]"
            formatted_content.append(formatted_text)
        return formatted_content

    def _get_current_formatted_date():
        """Helper function to get current formatted date"""
        weekdays = ['一', '二', '三', '四', '五', '六', '日']
        current_time = datetime.now()
        weekday = weekdays[current_time.weekday()]
        return f"{current_time.year}年{current_time.month:02d}月{current_time.day:02d}日星期{weekday}"

        # Parse input messages

    spark_msgs = json.loads(prompt)["messages"]

    # Find last tool message
    last_tool_msg = next((msg for msg in reversed(spark_msgs) if msg["role"] == "tool"), None)

    # Create messages list excluding tool messages
    openai_msgs = [{"role": item["role"], "content": item["content"]}
                   for item in spark_msgs if item["role"] != "tool"]

    # Process and format content if tool message exists
    if last_tool_msg and prompt_search_template:
        search_content = json.loads(last_tool_msg["content"])
        if search_content:
            # Find last user message and update its content
            for msg in reversed(openai_msgs):
                if msg["role"] == "user":
                    formatted_content = _format_search_content(search_content)
                    current_date = _get_current_formatted_date()

                    # Apply template
                    msg["content"] = prompt_search_template.format(
                        search_results="\n".join(formatted_content),
                        cur_date=current_date,
                        question=msg["content"]
                    )
                    break

    return openai_msgs


class ThreadPool:
    def __init__(self, wrapper):
        self.workers = int(os.environ.get("THREAD_POOL_WORKERS", min(300, os.cpu_count() * 4))) # 优化线程数
        self.threads = {}
        self.task_queues = {}
        self.lock = threading.Lock()
        self.wrapper = wrapper
        import logging
        self.filelogger = getFileLogger(level=logging.DEBUG)
        
        # 创建inference线程池
        self.inference_executor = ThreadPoolExecutor(max_workers=self.workers)
        
        # 创建worker线程和任务队列
        for i in range(self.workers):
            process_id = os.getpid()
            thread_id = "process-{}-thread-{}".format(str(process_id), str(i))
            task_queue = queue.Queue()  # 使用同步队列
            self.threads[thread_id] = threading.Thread(target=self.worker, args=(thread_id, task_queue))
            self.task_queues[thread_id] = task_queue
            self.threads[thread_id].start()

    def worker(self, thread_id, task_queue):
        self.filelogger.debug(f"task_loop {thread_id} enter")
        while True:
            try:
                task = task_queue.get()  # 同步获取任务
                if task is None:
                    self.filelogger.info(f"Thread {thread_id} received stop signal")
                    break
                self.filelogger.info(f"task {task.requestInfo.sid} enter")
                # 使用线程池执行inference
                self.inference_executor.submit(task.wrapper.openaiInference, task)
                task_queue.task_done()
            except Exception as e:
                self.filelogger.error(f"Error in task loop: {e}")
                continue

    def alloc_min_thread(self) -> str:
        with self.lock:
            min_thread_id = min(self.threads, key=lambda thread_id: self.task_queues[thread_id].qsize())
        return min_thread_id

    def put_task(self, _fn, thread_id, task):
        # 将任务放入指定线程的队列
        self.task_queues[thread_id].put(task)

    def wait_completion(self):
        # 发送停止信号给所有worker线程
        for task_queue in self.task_queues.values():
            task_queue.put(None)
        # 等待所有worker线程完成
        for thread in self.threads.values():
            thread.join()
        # 关闭inference线程池
        self.inference_executor.shutdown(wait=True)


# 定义服务推理逻辑
class Wrapper(WrapperBase):
    serviceId = "atp"
    version = "v1"

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.enable_metrics = False
        import logging
        self.filelogger = getFileLogger(level=logging.DEBUG)
        self.patch_id = {}
        self.patch_id_lock = threading.Lock()
        self.request_map: dict[str, RequestInfo] = {}
        self.request_map_lock = threading.Lock()
        self.thread_pool = None
        
        # 添加清理线程管理器
        self.cleanup_manager = CleanupThreadManager(self.filelogger)
        self.cleanup_manager.start()
        
        self.filelogger.info(f"vllm openai client wrapper constructed")

    def get_free_port(self):
        import socket
        # 创建一个临时的socket对象
        temp_socket = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        temp_socket.bind(('localhost', 0))  # 绑定到本地地址的一个随机可用端口
        _, port = temp_socket.getsockname()  # 获取分配的端口号
        temp_socket.close()  # 关闭临时socket
        return port

    def wait_server_ready(self, server_url: str):
        import requests
        while True:
            try:
                response = requests.get(server_url + "/health", timeout=(1, 3))
                # 检查响应状态码
                if response.status_code == 200:
                    self.filelogger.info(f"{server_url} ready, {response.content}")
                    break
                else:
                    self.filelogger.info(f"{server_url} not ready")
                response.close()
            except Exception as e:
                self.filelogger.info(f"{server_url} connect exception:{e}")
            time.sleep(5)

    def setupDefaultEnv(self):
        if os.environ.get("VLLM_CONFIGURE_LOGGING", None) is None:
            os.environ['VLLM_CONFIGURE_LOGGING'] = "1"

    def uploadLb(self, port):
        while True:
            try:
                data_upload = {}
                response = requests.get(f"http://localhost:{port}/metrics", timeout=(1, 3))
                if response.status_code == 200 and callback_lb_extra:
                    self.filelogger.debug("Uploading LB status: {}".format(response.status_code))

                    for family in text_string_to_metric_families(response.text):
                        for sample in family.samples:
                            data_upload[sample.name] = str(sample.value)
                    ret = callback_lb_extra({"status": "11111"})
                    print(ret)
                time.sleep(9)
                self.filelogger.info("Upload status ..................... {}".format(response.status_code))
            except Exception as e:
                time.sleep(9)
                print(str(e))
                continue

    def setupStatus(self, port):
        '''
        livenessProbe:
          exec:
            command:
              - /bin/sh
              - -c
              - "if [ -f /path/to/status ] && [ $(cat /path/to/status) -eq 0 ]; then exit 0; else exit 1; fi"
          initialDelaySeconds: 5
          periodSeconds: 10

        '''

        def run(port):
            if not os.path.exists("/var/run"):
                os.makedirs("/var/run")
            while True:
                time.sleep(3)
                with open("/var/run/wrapper_status", 'w') as h:
                    r = requests.get(f"http://localhost:{port}/health", timeout=(1, 3))
                    if r.status_code == 200:
                        h.write('0')
                    else:
                        h.write('-1')
                    h.close()

        thread = threading.Thread(target=run, args=(port,))
        thread.start()

    def monitor_subprocess(self, proc):
        self.filelogger.warn("Starting Sglang process healthy Monitor...")
        while True:
            retcode = proc.poll()
            if retcode is not None:
                self.filelogger.warn(f"Sglang 进程已退出，返回码: {retcode}")
                exit(retcode)
            time.sleep(1)

    # 线程函数: 检查 engine 是否健康
    def monitor_engine(self, port):
        self.filelogger.warn("Starting Sglang engine healthy Monitor...")
        if not os.path.exists("/var/run"):
            os.makedirs("/var/run")
        while True:
            with open("/var/run/wrapper_status", 'w') as h:
                if self.check_engine_health(port):
                    h.write('0')
                else:
                    h.write('-1')
                    self.filelogger.info("LLM Engine is not healthy. Retrying...")
            h.close()
            time.sleep(10)  # 等待 2 秒后重试

    def check_engine_health(self, port):
        retry = 2
        i = 0
        health = self.health
        while i <= retry:
            i += 1
            req = requests.get(f"http://localhost:{port}/health_generate", timeout=(1, 3))
            if req.status_code == 200:
                health = True
                break
            else:
                health = False
        self.health = health
        return self.health

    def get_local_ip(self):
        with socket.socket(socket.AF_INET, socket.SOCK_DGRAM) as s:
            s.connect(("1.1.1.1", 80))  # 连接外部服务器（不会真正发送数据）
            return s.getsockname()[0]

    def wrapperInit(self, config: {}) -> int:
        self.prompt_search_template = config.get('prompt_search_template')
        self.filelogger.info(f"wrapperInitConfig:{self.prompt_search_template}")
        self.setupDefaultEnv()
        self.base_model = os.environ.get("FULL_MODEL_PATH")
        self.pretrained_name = os.environ.get("PRETRAINED_MODEL_NAME")
        self.finetune_type = os.environ.get("FINETUNE_TYPE", 'lora')
        self.infer_type = os.environ.get("INFER_TYPE")
        self.pd_type = os.environ.get("AIGES_PD_ROLE", None)
        self.health = True
        port = os.environ.get("HTTP_SERVER_PORT", "40000")  # 通信接口
        port = int(port)
        self.http_server_port = int(port)
        self.filelogger.info(f"base_model:{self.base_model}")

        self.bootstrap_ip = self.get_local_ip()
        self.id_allocator = CircularIDAllocator(mock_ip=self.bootstrap_ip, port=self.http_server_port)
        self.filelogger.info(f"Getting local IP... {self.bootstrap_ip}")

        self.model_generation_config: transformers.GenerationConfig = None
        if os.path.exists(os.path.join(self.base_model, "generation_config.json")):
            self.filelogger.info("Loading Default Generation Config")
            self.model_generation_config = GenerationConfig.from_pretrained(self.base_model)

        enable_metrics = os.environ.get("ENABLE_METRICS", None)
        if enable_metrics and enable_metrics.lower() == "true":
            self.enable_metrics = True

        timeout = os.environ.get("READ_TIMEOUT", "30")
        dist_port = os.environ.get("DIST_PORT", "20000")  # 分布式通信接口
        dist_port = int(dist_port)


        if not self.pretrained_name:
            self.filelogger.error("should have environ(PRETRAINED_MODEL_NAME,FULL_MODEL_PATH(lora weight dir))")
            return -1

        if not os.path.isdir(self.base_model):
            self.filelogger.error(f"not find the base_model in FULL_MODEL_PATH")
            return -1

        ## 获取服务监听端口
        if port == 0:
            port = self.get_free_port()

        self.filelogger.info("will using {} port, dist server port: {}".format(port, dist_port))

        ## 服务器实际地址
        serverUrl = f"http://127.0.0.1:{port}/v1"
        self.setupStatus(port)

        ## 启动服务器进程
        sg_proc = launch_openai_server(self.pretrained_name, self.base_model, port, dist_port, self.filelogger,
                                       self.pd_type, self.enable_metrics)

        # mon_thread = threading.Thread(target=self.monitor_engine, args=(port,))
        mon_thread = threading.Thread(target=self.monitor_subprocess, args=(sg_proc,))
        mon_thread.start()

        if self.enable_metrics:
            lbthread = threading.Thread(target=self.uploadLb, args=(port,))
            lbthread.start()

        ## 监听服务器是否启动完成
        self.server_url = serverUrl
        ## 创建openai客户端
        self.client = OpenAI(base_url=serverUrl, api_key="maas", timeout=float(timeout))
        self.asyncClient = AsyncOpenAI(base_url=serverUrl, api_key="maas", timeout=float(timeout))

        self.thread_pool = ThreadPool(wrapper=self)
        self.filelogger.info(f'wrapper init success')
        return 0

    def checkValidLora(self, pretrained_name, lora_path):
        import json
        self.filelogger.debug(f'check lora {pretrained_name}, {lora_path}')
        confjson = "adapter_config.json"
        if not os.path.isdir(lora_path):
            msg = "not find  %s" % lora_path
            self.filelogger.error(msg)
            return False, msg
        files = os.listdir(lora_path)
        if not confjson in files:
            msg = "%s doesnt have file adapter_config.json" % lora_path
            self.filelogger.error(msg)
            return False, msg
        fp = open(os.path.join(lora_path, confjson), "rb")
        conf = json.load(fp)
        base_model_path = conf.get("base_model_name_or_path", "")
        if not base_model_path:
            msg = "config json not contains base_model_name_or_path...c=" % lora_path
            self.filelogger.error(msg)
            return False, msg
        self.filelogger.debug(f'base_model_path:{base_model_path}')
        if pretrained_name not in base_model_path:
            msg = f"current runntime model is {pretrained_name}, but you pass the {base_model_path}"
            self.filelogger.error(msg)
            return False, msg
        return True, "Check Success..."

    def lora_load(self):
        record_path = "/home/.atp/lora_record"
        lora_weight_path = "/home/.atp/lora_weight/"
        if not os.path.isdir(record_path):
            self.filelogger.debug("not need load lora now ")
            return
        patch_ids = os.listdir(record_path)
        paths = os.listdir(lora_weight_path)
        for patch_id in patch_ids:
            if not patch_id in paths:
                self.filelogger.warn("not find this %s" % patch_id)
                continue
            lora_weight_path_one = lora_weight_path + patch_id
            valid, msg = self.checkValidLora(self.pretrained_name, lora_weight_path_one)
            if not valid:
                self.filelogger.warn("not valid lora for this model. %s" % patch_id)
                continue
            self.patch_id_lock.acquire()
            self.patch_id[patch_id] = lora_weight_path_one
            self.patch_id_lock.release()
            self.filelogger.info(f'success wrapperLoadRes patch_id {patch_id}')

    def wrapperLoadRes(self, reqData: DataListCls, patch_id: int) -> int:
        self.patch_id_lock.acquire()
        if patch_id in self.patch_id:
            self.filelogger.warn("patch_id has exist.Please first to UnloadRes")
            self.patch_id_lock.release()
            return 0
        self.patch_id_lock.release()

        lora_weight_path = "/home/.atp/lora_weight/"
        lora_weight_path = os.path.join(lora_weight_path, str(patch_id))
        if os.path.exists(lora_weight_path):
            self.filelogger.warn("zip file has exist.Please first to UnloadRes")

        import io
        import zipfile

        byte_stream = io.BytesIO(reqData.list[0].data)
        # 解压缩 zip 文件到指定目录
        with zipfile.ZipFile(byte_stream, "r") as zip_ref:
            zip_ref.extractall(lora_weight_path)

        # write reocrd
        record_path = "/home/.atp/lora_record"
        rf = open(os.path.join(record_path, patch_id), "w")
        rf.close()

        valid, msg = self.checkValidLora(self.pretrained_name, lora_weight_path)
        if not valid:
            return -1
        self.filelogger.debug(msg)

        # lora加载
        load_resp = post_with_retry(
            url=self.server_url + "/load_lora_adapter",
            json={"lora_name": patch_id, "lora_path": lora_weight_path, "base_model_name": self.pretrained_name}
        )
        self.filelogger.info(f"patch_id:{patch_id}, load_resp:{load_resp}")

        self.patch_id_lock.acquire()
        self.patch_id[patch_id] = lora_weight_path
        self.patch_id_lock.release()
        self.filelogger.debug("Load Resource Successfully...")
        return 0

    def wrapperUnloadRes(self, presid: int) -> int:
        self.patch_id_lock.acquire()
        if presid not in self.patch_id:
            self.filelogger.error("patch_id not exist")
            self.patch_id_lock.release()
            return 0
        lora_weight_path = self.patch_id[presid]
        if not os.path.exists(lora_weight_path):
            self.filelogger.error("lora weigth path not exist")
            self.patch_id_lock.release()
            return 0

        unload_resp = post_with_retry(
            url=self.server_url + "/unload_lora_adapter",
            json={"lora_name": presid}
        )
        self.filelogger.info(f"patch_id:{presid}, unload_resp:{unload_resp}")

        import shutil
        # 递归地删除目录及其内容
        shutil.rmtree(lora_weight_path)
        del self.patch_id[presid]
        self.patch_id_lock.release()
        return 0

    def wrapperOnceExec(self, params: {}, reqData: DataListCls, usrTag: str = "", persId: int = 0) -> Response:
        prompt = get_payload_messages(reqData)
        functions = get_payload_functions(reqData)

        requestInfo = RequestInfo("", params)

        thread_id = self.thread_pool.alloc_min_thread()
        result_q = queue.Queue()
        inferenceInfo = PromptInferenceInfo(self, thread_id, RequestMode.ONCE, prompt, requestInfo, functions, result_q)
        self.filelogger.info(f'start wrapperOnceExec params: {params}, sid:{inferenceInfo.request_id}')

        self.thread_pool.put_task(self.openaiInference, thread_id, inferenceInfo)

        try:
            res = result_q.get(timeout=12)
        except queue.Empty:
            res = Response()
            content = resp_content(Once, 0, "wrapper.timeout.")
            usage = resp_usage(Once, 0, 0)
            res.list = [content, usage]

        self.filelogger.debug(
            f'success wrapperOnceExec,params:{params},prompt:{prompt},request_id:{inferenceInfo.request_id}')
        return res

    def wrapperOnceExecAsync(self, params: {}, reqData: DataListCls, usrTag: str, persId: int = 0):
        prompt = get_payload_messages(reqData)
        functions = get_payload_functions(reqData)
        if functions:
            self.filelogger.debug(f'start wrapperOnceExec functions: size {len(functions)}')
        requestInfo = RequestInfo("", params, usrTag)

        thread_id = self.thread_pool.alloc_min_thread()
        inferenceInfo = PromptInferenceInfo(self, thread_id, RequestMode.ONCE_ASYNC, prompt, requestInfo, functions)
        self.filelogger.info(f'start wrapperOnceExecAsync params:{params},request_id:{inferenceInfo.request_id}')

        self.thread_pool.put_task(thread_id, inferenceInfo)

        self.filelogger.debug(
            f'success wrapperOnceExecAsync,params:{params},prompt:{prompt},request_id:{inferenceInfo.request_id}')
        return 0

    def wrapperFini(self) -> int:
        # 停止清理管理器
        self.cleanup_manager.stop()
        self.thread_pool.wait_completion()
        return 0

    def wrapperError(cls, ret: int) -> str:
        if ret == 1002:
            return "Engine is busy..."
        if ret == 1001:
            return "Engine error"
        if ret == 1003:
            return "TimeOut"
        if ret == 1004:
            return "wrong data input.."
        return ""

    def wrapperWrite(self, handle: str, req: DataListCls) -> int:
        self.filelogger.debug(f'start wrapperWrite handle {handle}')
        room_id: int = 0
        bootstrap_ip: str = ""
        bootstrap_port: int = int(os.environ.get("PD_BOOTSTRAP_PORT", "8998"))
        prefill_addr: str = ""
        is_decode = True
        if self.pd_type == "prefill":
            is_decode = False

        messages = req.get("messages")

        if not is_decode:
            room_id = self.id_allocator.allocate()
            bootstrap_ip = self.bootstrap_ip
            prefill_addr = f'{bootstrap_ip}:{self.http_server_port}'

            self.filelogger.debug(f'prefill room id:{room_id} {bootstrap_ip}:{bootstrap_port}')
        else:
            self.filelogger.info(f"decode messages len： {len(messages.data)}")
            messages.status = DataEnd  # 强制设置 p发来的message 为 data end
            kv_info_raw = req.get("__kv_info")
            if kv_info_raw:
                kv_info = kv_info_raw.data.decode("utf-8")
                self.filelogger.debug(f'decode getting the kv_info:{kv_info}')
                kv_info = json.loads(kv_info)
                room_id = kv_info.get("bootstrap_room")
                bootstrap_ip = kv_info.get("bootstrap_ip")
                prefill_addr = kv_info.get("prefill_addr")
                bootstrap_port = kv_info.get("bootstrap_port")

        self.filelogger.debug(f'getting messages:{handle}: {messages}')
        if messages:
            messages_data = messages.data
            self.request_map_lock.acquire()
            requestInfo = self.request_map[handle]
            self.request_map_lock.release()
            requestInfo.requests.append(messages_data)
            # 设置  pd信息
            requestInfo.bootstrap_room = room_id
            requestInfo.prefill_bootstrap_ip = bootstrap_ip
            requestInfo.prefill_addr = prefill_addr
            requestInfo.bootstrap_port = bootstrap_port

            messages_status = messages.status
            if messages_status == DataEnd:
                messages_bytes = b''.join(requestInfo.requests)
                prompt = messages_bytes.decode('utf-8')
                new_messages = json.loads(prompt)
                # function_call
                if isinstance(new_messages, list):
                    self.filelogger.info(f'Wrong data format:  {new_messages}')
                    return 1004
                # function_call
                functions = new_messages.get("functions", None)
                if functions:
                    self.filelogger.info(f'start wrapperWrite functions: size {len(functions)}')
                thread_id = self.thread_pool.alloc_min_thread()
                inferenceInfo = PromptInferenceInfo(self, thread_id, RequestMode.STREAM, prompt, requestInfo, functions,
                                                    messages=messages_bytes)
                self.thread_pool.put_task(self.openaiInference, thread_id, inferenceInfo)
                requestInfo.requests.append(inferenceInfo.request_id)  # todo 分析下为啥？
                self.filelogger.debug(
                    f'success wrapperWrite sid:{requestInfo.sid} handle:{handle}')
        return 0

    def wrapperCreate(self, params: {}, sid: str, persId: int = 0, usrTag: str = "") -> SessionCreateResponse:
        patch_id = str(params.get('patch_id', "0"))
        if len(patch_id) == 0:
            patch_id = "0"
        self.filelogger.info(f'start wrapperCreate {params} sid: {sid}')
        requestInfo = RequestInfo(sid, params, usrTag)
        self.request_map_lock.acquire()
        self.request_map[requestInfo.handle] = requestInfo
        self.request_map_lock.release()

        s = SessionCreateResponse()
        s.handle = requestInfo.handle
        s.error_code = 0
        self.filelogger.debug(f'success wrapperCreate {patch_id}, sid: {sid}')
        return s

    def wrapperDestroy(self, handle: str) -> int:
        self.filelogger.info(f'start wrapperDestroy {handle}')
        self.request_map_lock.acquire()
        requestInfo = self.request_map[handle]
        if requestInfo == None:
            self.filelogger.error("can't get this handle:" % handle)
            self.request_map_lock.release()
            return 0
        sid = requestInfo.sid
        self.request_map_lock.release()
        requestInfo.stop_q.put(True, block=False)
        self.request_map_lock.acquire()
        del self.request_map[handle]
        self.request_map_lock.release()

        self.filelogger.info(f'success wrapperDestroy {handle} sid: {sid}')
        return 0

    def wrapperTestFunc(cls, data: [], respData: []):
        pass

    def abort_mon_thread(self, user_stop_q, respStream, sid):
        def abort(user_stop_q, respStream, sid, event):
            stop = False
            while not stop:
                if not user_stop_q.empty():
                    user_stoped = user_stop_q.get_nowait()
                    if user_stoped:
                        self.filelogger.info(f'====>sync inference end, {sid}')
                        try:
                            respStream.close()
                        except Exception as e:
                            self.filelogger.warning(f"Error closing respStream: {e}")
                        break
                if event.is_set():
                    self.filelogger.info(f'====>end abort thread normal, {sid}')
                    break

                time.sleep(0.1)  # 减少轮询间隔从1秒到0.1秒

        event = threading.Event()
        thread = threading.Thread(target=abort, args=(user_stop_q, respStream, sid, event))
        thread.daemon = True  # 设置为守护线程，主线程结束时自动退出
        thread.start()
        return event

    def cleanup_monitor_thread(self, stop_mon_event, streamResp, requestInfo):
        """将清理任务提交给清理管理器"""
        self.cleanup_manager.add_cleanup_task(stop_mon_event, streamResp, requestInfo)

    def openaiInference(self, inferenceInfo: PromptInferenceInfo):
        requestInfo = inferenceInfo.requestInfo
        self.filelogger.info(f'====>sync inference start, {requestInfo.sid}')

        # Determine if this is decode mode based on pd_type
        is_decode = True
        if self.pd_type == "prefill":
            is_decode = False

        request_id = inferenceInfo.request_id
        user_tag = requestInfo.user_tag
        sid = requestInfo.sid
        is_stoped = False
        if not requestInfo.stop_q.empty():
            is_stoped = requestInfo.stop_q.get_nowait()
        if is_stoped:
            state = Once
            if inferenceInfo.mode == RequestMode.STREAM:
                state = DataEnd
            content = resp_content(state, 0, ' ')
            usage = resp_usage(state, 0, 0)
            res = Response()
            res.list = [content, usage]
            if (inferenceInfo.mode == RequestMode.STREAM or
                    inferenceInfo.mode == RequestMode.ONCE_ASYNC):
                if not is_decode:
                    callback(res, user_tag)
            elif inferenceInfo.mode == RequestMode.ONCE:
                inferenceInfo.result_q.put(res)

            keep_alive_data = resp_keepalive_down(DataEnd)
            res = Response()
            res.list = [keep_alive_data]
            ret = callback(res, requestInfo.user_tag)
            self.filelogger.info(f"prefill end... release callback {ret} ")

            self.filelogger.info(f'====>inference abort before infer, {requestInfo.sid}')
            return

        params = requestInfo.params
        patch_id = str(params.get('patch_id', '0'))
        if len(patch_id) == 0:
            patch_id = "0"
        prompt = inferenceInfo.prompt

        model_name = self.pretrained_name
        lora_path = None
        if self.finetune_type == 'lora' or self.finetune_type == 'qlora':
            if patch_id != '0' and patch_id != 0:
                self.patch_id_lock.acquire()
                lora_path = self.patch_id.get(patch_id)
                self.patch_id_lock.release()
                if lora_path == None:
                    self.filelogger.error(
                        f"lora or qlora infer, but lora path is invalid for patch_id:{patch_id}, will use base infer instead")
                else:
                    model_name = patch_id

        ## openai接口不支持top_k和repetition_penalty
        repetition_penalty = get_param_repetition_penalty(params)
        temperature = get_param_temperature(params)
        if IS_REASONING_MODEL and R1_FIXED_TEMPERATURE is not None:
            temperature = float(R1_FIXED_TEMPERATURE)
            self.filelogger.info(
                f"Using R1_FIXED_TEMPERATURE set in ENVIRONMENT variable: {temperature}")

        top_p = get_parma_top_p(params)
        # generation_config
        if self.model_generation_config:
            top_k = self.model_generation_config.top_k
            top_p = self.model_generation_config.top_p
            do_sample = self.model_generation_config.do_sample
            temperature = self.model_generation_config.temperature
            repetition_penalty = self.model_generation_config.repetition_penalty

        max_tokens = get_param_max_new_tokens(params)
        top_k = get_parma_tok_k(params)

        openaiMsgs = format_messages(prompt, self.prompt_search_template)
        self.filelogger.info(f"sid:{sid},sparkMsgs:{openaiMsgs}")

        isFc = (inferenceInfo.functions is not None and len(inferenceInfo.functions) > 0)
        full_content = ""
        reasoning_content = ""
        prompt_tokens_len = 0
        result_tokens_len = 0

        stop_mon_event = None
        index = 0

        try:
            ## 如果是普通流式请求
            thinking = False
            if IS_REASONING_MODEL:
                lastMsg = openaiMsgs[-1]
                if lastMsg["role"] != "assistant":
                    openaiMsgs += [{"role": "assistant", "content": "<think>\n"}]
                    thinking = True

            # 使用mock数据
            mock_content = "这是一个模拟的流式响应。我们将逐步返回这个响应，以模拟真实的流式输出效果。每个chunk都会包含一部分内容，直到完整返回所有内容。"
            streamResp = MockStreamResponse(content=mock_content, chunk_size=10)

            # Add PD related response handling
            if not is_decode:
                kv_info = resp_kv_info(DataEnd, requestInfo.prefill_bootstrap_ip, requestInfo.bootstrap_room,
                                     requestInfo.prefill_addr, requestInfo.bootstrap_port)
                p_messages = resp_p_messages(DataEnd, inferenceInfo.messages)
                self.filelogger.info(f"====>len prefill {len(inferenceInfo.messages)}")
                keep_alive_data = resp_keepalive_down(DataBegin)
                res = Response()
                res.list = [kv_info, p_messages, keep_alive_data]
                ret = callback(res, requestInfo.user_tag)
                if ret == 0:
                    self.filelogger.info(
                        f'====> prefill first callback success, {requestInfo.prefill_bootstrap_ip}, {requestInfo.bootstrap_room}')
                else:
                    self.filelogger.info(
                        f'====> prefill first callback error, {requestInfo.prefill_bootstrap_ip}, {requestInfo.bootstrap_room}')

            state = DataNone
            

            if  is_decode:
                # 添加 abort 线程
                stop_mon_event = self.abort_mon_thread(requestInfo.stop_q, streamResp, requestInfo.sid)
                for chunk in streamResp:
                    if len(chunk.choices) > 0 and chunk.choices[0].delta.content is not None:
                        chunk_content = str(chunk.choices[0].delta.content)
                        res = Response()
                        if state == DataNone:
                            state = DataBegin
                        elif state == DataBegin:
                            state = DataContinue
                        if chunk_content == R1_THINK_START:
                            thinking = True
                            continue
                        if R1_THINK_END in str(chunk_content):
                            thinking = False
                            reasoning_last_chunk, answer_start_chunk = chunk_content.split(R1_THINK_END)
                            content = resp_content(state, index, "", reasoning_content=reasoning_last_chunk)
                            full_content = full_content + str(reasoning_last_chunk)
                            res.list = [content]
                            callback(res, user_tag)
                            index += 1

                            content = resp_content(state, index, answer_start_chunk, reasoning_content="")
                            full_content = full_content + str(answer_start_chunk)
                            res.list = [content]
                            callback(res, user_tag)
                            index += 1
                            continue
                        if thinking:
                            content = resp_content(state, index, "", reasoning_content=chunk_content)
                        else:
                            content = resp_content(state, index, chunk_content, reasoning_content="")

                        full_content = full_content + str(chunk_content)
                        res.list = [content]
                        callback(res, user_tag)
                        index += 1
                    if chunk.usage is not None:
                        prompt_tokens_len = chunk.usage.prompt_tokens
                        result_tokens_len = chunk.usage.completion_tokens

            requestInfo.stop_q.put(True, block=False)

        except Exception as e:
            traceback.print_exc()
            self.filelogger.error(f"An error occurred when infer: {e}")
            res = Response()
            res = res.response_err(1001)
            callback(res, user_tag)
        finally:
            if is_decode:
                response_end(index, user_tag, prompt_tokens_len, result_tokens_len)
            else:
                keep_alive_data = resp_keepalive_down(DataEnd)
                res = Response()
                res.list = [keep_alive_data]
                ret = callback(res, requestInfo.user_tag)
                self.filelogger.info(f"prefill end... release callback {ret} ")
            self.cleanup_monitor_thread(stop_mon_event, streamResp, requestInfo)
            self.filelogger.info(
                f'====>streaming inference end, {requestInfo.sid}:{full_content}, sid:{sid},in_tokens:{prompt_tokens_len}, out_tokens:{result_tokens_len}')
            return


def response_end(index, user_tag, prompt_tokens_len, result_tokens_len):
    state = DataEnd
    content = resp_content(state, index, ' ')
    usage = resp_usage(state, prompt_tokens_len, result_tokens_len)
    res = Response()
    res.list = [content, usage]
    callback(res, user_tag)


def generate_template(t="alpaca"):
    template = "/home/aiges/default.jinja"
    with open(template, 'w') as f:
        if t == "alpaca":
            f.write('''{{ (messages|selectattr('role', 'equalto', 'system')|list|last).content|trim if (messages|selectattr('role', 'equalto', 'system')|list) else '' }}

{% for message in messages %}
{% if message['role'] == 'user' %}
### Instruction:
{{ message['content']|trim -}}
{% if not loop.last %}


{% endif %}
{% elif message['role'] == 'assistant' %}
### Response:
{{ message['content']|trim -}}
{% if not loop.last %}


{% endif %}
{% elif message['role'] == 'user_context' %}
### Input:
{{ message['content']|trim -}}
{% if not loop.last %}


{% endif %}
{% endif %}
{% endfor %}
{% if add_generation_prompt and messages[-1]['role'] != 'assistant' %}
### Response:
{% endif %}''')
        elif t == "llama2_json":
            f.write('''{{- bos_token }}
{%- if custom_tools is defined %}
    {%- set tools = custom_tools %}
{%- endif %}
{%- if not tools_in_user_message is defined %}
    {%- set tools_in_user_message = false %}
{%- endif %}
{%- if not date_string is defined %}
    {%- if strftime_now is defined %}
        {%- set date_string = strftime_now("%d %b %Y") %}
    {%- else %}
        {%- set date_string = "26 Jul 2024" %}
    {%- endif %}
{%- endif %}
{%- if not tools is defined %}
    {%- set tools = none %}
{%- endif %}

{#- Find out if there are any images #}
{% set image_ns = namespace(has_images=false) %}
{%- for message in messages %}
    {%- for content in message['content'] %}
        {%- if content['type'] == 'image' %}
            {%- set image_ns.has_images = true %}
        {%- endif %}
    {%- endfor %}
{%- endfor %}

{#- This block extracts the system message, so we can slot it into the right place. #}
{%- if messages[0]['role'] == 'system' %}
    {%- if messages[0]['content'] is string %}
        {%- set system_message = messages[0]['content']|trim %}
    {%- else %}
        {%- set system_message = messages[0]['content'][0]['text']|trim %}
    {%- endif %}
    {%- set messages = messages[1:] %}
{%- else %}
    {%- if tools is not none %}
        {%- set system_message = "You are a helpful assistant with tool calling capabilities. Only reply with a tool call if the function exists in the library provided by the user. If it doesn't exist, just reply directly in natural language. When you receive a tool call response, use the output to format an answer to the original user question." %}
    {%- else %}
        {%- set system_message = "" %}
    {%- endif %}
{%- endif %}

{#- System message if there are no images, if the user supplied one, or if tools are used (default tool system message) #}
{%- if system_message or not image_ns.has_images %}
    {{- "<|start_header_id|>system<|end_header_id|>\n\n" }}
    {%- if tools is not none %}
        {{- "Environment: ipython\n" }}
    {%- endif %}
    {{- "Cutting Knowledge Date: December 2023\n" }}
    {{- "Today Date: " + date_string + "\n\n" }}
    {%- if tools is not none and not tools_in_user_message %}
        {{- "You have access to the following functions. To call a function, please respond with JSON for a function call. " }}
        {{- 'Respond in the format {"name": function name, "parameters": dictionary of argument name and its value}. ' }}
        {{- "Do not use variables.\n\n" }}
        {%- for t in tools %}
            {{- t | tojson(indent=4) }}
            {{- "\n\n" }}
        {%- endfor %}
    {%- endif %}
    {{- system_message }}
    {{- "<|eot_id|>" }}
{%- endif %}

{#- Custom tools are passed in a user message with some extra guidance #}
{%- if tools_in_user_message and not tools is none %}
    {#- Extract the first user message so we can plug it in here #}
    {%- if messages | length != 0 %}
        {%- if messages[0]['content'] is string %}
            {%- set first_user_message = messages[0]['content']|trim %}
        {%- else %}
            {%- set first_user_message = messages[0]['content'] | selectattr('type', 'equalto', 'text') | map(attribute='text') | map('trim') | join('\n') %}
        {%- endif %}
        {%- set messages = messages[1:] %}
    {%- else %}
        {{- raise_exception("Cannot put tools in the first user message when there's no first user message!") }}
    {%- endif %}
    {{- '<|start_header_id|>user<|end_header_id|>\n\n' -}}
    {{- "Given the following functions, please respond with a JSON for a function call " }}
    {{- "with its proper arguments that best answers the given prompt.\n\n" }}
    {{- 'Respond in the format {"name": function name, "parameters": dictionary of argument name and its value}. ' }}
    {{- "Do not use variables.\n\n" }}
    {%- for t in tools %}
        {{- t | tojson(indent=4) }}
        {{- "\n\n" }}
    {%- endfor %}
    {{- first_user_message + "<|eot_id|>"}}
{%- endif %}

{%- for message in messages %}
    {%- if not (message.role == 'ipython' or message.role == 'tool' or 'tool_calls' in message) %}
        {{- '<|start_header_id|>' + message['role'] + '<|end_header_id|>\n\n' }}
        {%- if message['content'] is string %}
            {{- message['content'] | trim}}
        {%- else %}
            {%- for content in message['content'] %}
                {%- if content['type'] == 'image' %}
                    {{- '<|image|>' }}
                {%- elif content['type'] == 'text' %}
                    {{- content['text'] | trim }}
                {%- endif %}
            {%- endfor %}
        {%- endif %}
        {{- '<|eot_id|>' }}
    {%- elif 'tool_calls' in message %}
        {%- if not message.tool_calls|length == 1 %}
            {{- raise_exception("This model only supports single tool-calls at once!") }}
        {%- endif %}
        {%- set tool_call = message.tool_calls[0].function %}
        {{- '<|start_header_id|>assistant<|end_header_id|>\n\n' -}}
        {{- '{"name": "' + tool_call.name + '", ' }}
        {{- '"parameters": ' }}
        {{- tool_call.arguments | tojson }}
        {{- "}" }}
        {{- "<|eot_id|>" }}
    {%- elif message.role == "tool" or message.role == "ipython" %}
        {{- "<|start_header_id|>ipython<|end_header_id|>\n\n" }}
        {%- if message.content is string %}
            {{- { "output": message.content } | tojson }}
        {%- else %}
            {%- for content in message['content']  %}
                {%- if content['type']  == 'text' %}
                    {{- { "output": content['text']  } | tojson }}
                {%- endif %}
            {%- endfor %}
        {%- endif %}
        {{- "<|eot_id|>" }}
    {%- endif %}
{%- endfor %}
{%- if add_generation_prompt %}
    {{- '<|start_header_id|>assistant<|end_header_id|>\n\n' }}
{%- endif %}''')

    return template

class CleanupThreadManager:
    def __init__(self, filelogger):
        self.filelogger = filelogger
        self.cleanup_queue = queue.Queue()
        self.running = True
        self.cleanup_thread = None

    def start(self):
        """启动清理线程"""
        self.cleanup_thread = threading.Thread(target=self._cleanup_loop)
        self.cleanup_thread.daemon = True
        self.cleanup_thread.start()
        self.filelogger.info("Cleanup thread manager started")

    def stop(self):
        """停止清理线程"""
        self.running = False
        if self.cleanup_thread:
            self.cleanup_thread.join(timeout=5)
            self.filelogger.info("Cleanup thread manager stopped")

    def add_cleanup_task(self, stop_mon_event, streamResp, requestInfo):
        """添加清理任务到队列"""
        self.cleanup_queue.put((stop_mon_event, streamResp, requestInfo))

    def _cleanup_loop(self):
        """清理线程的主循环"""
        while self.running:
            try:
                # 非阻塞方式获取任务
                try:
                    stop_mon_event, streamResp, requestInfo = self.cleanup_queue.get_nowait()
                except queue.Empty:
                    time.sleep(0.1)  # 短暂休眠避免CPU空转
                    continue

                try:
                    # 先设置事件，通知监控线程停止
                    stop_mon_event.set()
                    
                    # 确保respStream被关闭
                    try:
                        streamResp.close()
                    except Exception as e:
                        self.filelogger.warning(f"Error closing streamResp: {e}")
                    
                    # 等待监控线程结束，但设置超时
                    try:
                        stop_mon_event.wait(timeout=0.5)  # 最多等待0.5秒
                    except Exception as e:
                        self.filelogger.warning(f"Error waiting for monitor thread: {e}")
                    
                    # 如果超时，强制清理
                    if not stop_mon_event.is_set():
                        self.filelogger.warning("Monitor thread did not stop in time, forcing cleanup")
                        try:
                            requestInfo.stop_q.put(True, block=False)
                        except Exception as e:
                            self.filelogger.warning(f"Error sending stop signal: {e}")
                except Exception as e:
                    self.filelogger.error(f"Error in cleanup: {e}")
                finally:
                    self.cleanup_queue.task_done()

            except Exception as e:
                self.filelogger.error(f"Error in cleanup loop: {e}")

class MockStreamResponse:
    def __init__(self, content="这是一个模拟的流式响应。", chunk_size=5):
        self.content = content
        self.chunk_size = chunk_size
        self.current_pos = 0
        self.usage = None
        self.choices = [type('Choice', (), {'delta': type('Delta', (), {'content': None})()})()]
        
    def __iter__(self):
        return self
        
    def __next__(self):
        if self.current_pos >= len(self.content):
            # 最后一次返回usage信息
            if not self.usage:
                self.usage = type('Usage', (), {'prompt_tokens': 10, 'completion_tokens': len(self.content)})()
                return self
            raise StopIteration
            
        chunk = self.content[self.current_pos:self.current_pos + self.chunk_size]
        self.current_pos += self.chunk_size
        self.choices[0].delta.content = chunk
        return self
        
    def close(self):
        pass
